#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>

typedef struct
{
    char lastname[21];
    char firstname[41];
    unsigned long birthdate;
} RecT;

int cmp(const void *a, const void *b)
{
  RecT *ap = (RecT *)a;
  RecT *bp = (RecT *)b;
  int ret = strcmp(ap->lastname, bp->lastname);

  if (ret == 0) return ap->birthdate - bp->birthdate;
  return ret;
}

void sortFile(const char *fName)
{
    RecT *r;
    FILE *f, *fOut;
    struct stat statBuf;
    int n;
    if (stat(fName, &statBuf) < 0)
    {
        perror(fName);
        exit(2);
    };
    n = statBuf.st_size / sizeof(RecT);
    r = (RecT *)calloc(n, sizeof(RecT));
    if (!(f = fopen(fName, "rb")))
    {
        perror(fName);
        exit(1);
    }
    if (!(fOut = fopen("tmp", "wb")))
    {
        perror("tmp");
        exit(1);
    }
    fread(r, sizeof(RecT), n, f);
    qsort(r, n, sizeof(RecT), cmp);
    fwrite(r, sizeof(RecT), n, fOut);
    fclose(f);
    fclose(fOut);
}

void printFile(const char *name)
{
    FILE *f;
    RecT rec;
    if (!(f = fopen(name, "rb")))
    {
        perror(name);
        exit(1);
    }
    while (fread(&rec, sizeof(RecT), 1, f))
      printf("%s %s, %lu\n", rec.lastname, rec.firstname, rec.birthdate);
    fclose(f);
    remove(name);
}

int main(int argc, char *argv[])
{
    FILE *f, *fOut;
    char buf[256], fName[256];
    RecT rec;
    struct stat statBuf;

    if (argc != 2)
    {
        printf("Usage %s fileName\nwhere fileName is the name of the file to with person data\n",
               argv[0]);
        exit(1);
    }
    strcpy(fName, argv[1]);
    if (stat(argv[1], &statBuf) < 0)
    {
        perror(argv[1]);
        exit(2);
    }
    if (!S_ISREG(statBuf.st_mode))
    {
        fprintf(stderr, "%s takes only regular files\n", argv[0]);
        exit(3);
    }
    if (!(f = fopen(argv[1], "r")))
    {
        perror(argv[1]);
        exit(1);
    }
    strcat(fName, ".dat");
    if (!(fOut = fopen(fName, "wb")))
    {
        perror(argv[1]);
        exit(1);
    }
    while (fgets(buf, sizeof(buf), f))
    {
        int i, j;
        char *token[10], *pos = strchr(buf, ',');


        if (!pos)
        {
            fprintf(stderr, "Bad input: %s\n", buf);
            continue;
        }
        else *pos = '\0';
        memset(&rec, '\0', sizeof(rec));
        for (i = 1, token[0] = strtok(buf, " ");
                (token[i] = strtok(NULL, " ")); i++);
        strcpy(rec.lastname, token[--i]);
        for (j = 0; j < i; j++)
        {
            strcat(rec.firstname, token[j]);
            if (j < i - 1) strcat(rec.firstname, " ");
        }
        int mon, day, year;
        for (i = 1, token[0] = strtok(pos+1, "/"); (token[i] = strtok(NULL, "/"));
                i++);
        mon = atoi(token[0]);
        day = atoi(token[1]);
        year = atoi(token[2]);
        rec.birthdate = year * 10000 + mon *100 + day;
        fwrite(&rec, sizeof(rec), 1, fOut);
    }
    fclose(f);
    fclose(fOut);
    sortFile(fName);
    printFile("tmp");
    return 0;

}
